@@ -31,7 +31,9 @@ Client*
 toolbar_client_new(Wm *w, Window win)
 {
   Window trans_win;
-  Client *c = base_client_new(w, win), *trans_client = NULL; 
+  Client *c = NULL, *trans_client = NULL; 
+
+  c = base_client_new(w, win);
 
   if (!c) return NULL;
 
@@ -46,19 +48,26 @@ toolbar_client_new(Wm *w, Window win)
   c->destroy      = &toolbar_client_destroy;
 
   XGetTransientForHint(w->dpy, win, &trans_win);
+
+  dbg("%s() checking trans hint\n", __func__);
    
   if (trans_win && (trans_win != win))
     {
+      dbg("%s() got trans hint\n", __func__);
+
       trans_client = wm_find_client(w, trans_win, WINDOW);
 
       if (trans_client)
 	{
+
 	  if (trans_client->type == MBCLIENT_TYPE_DIALOG)
 	    {
 	      c->flags |= CLIENT_TB_ALT_TRANS_FOR_DIALOG;
+	      dbg("%s() is trans for dialog\n", __func__);
 	    }
 	  else if (trans_client->type & (MBCLIENT_TYPE_APP|MBCLIENT_TYPE_DESKTOP))
 	    {
+	      dbg("%s() is trans for app\n", __func__);
 	      c->flags |= CLIENT_TB_ALT_TRANS_FOR_APP;
 	    }
 	  else trans_client = NULL;
@@ -83,11 +92,16 @@ toolbar_win_offset(Client *c)
 void
 toolbar_client_configure(Client *c)
 {
-  Wm *w = c->wm;
+  Wm     *w = c->wm;
+  Client *main_client = NULL;
+
+  dbg("%s() called\n", __func__);
 
   if (c->flags & CLIENT_IS_MINIMIZED)
     return;
 
+  dbg("%s() client is not minimised\n", __func__);
+
   if (c->flags & CLIENT_TB_ALT_TRANS_FOR_APP)
     {
       /* resize the main client were transient for */
@@ -103,36 +117,68 @@ toolbar_client_configure(Client *c)
 	}
     }
 
-  if (c->flags & CLIENT_TB_ALT_TRANS_FOR_DIALOG)
-    {
-      Client *dialog_client = c->trans;
-      
-      if (dialog_client)
-	{
-	  /*
-	   *  Move transient dialog out of the way.  
-	   */
-	  int req_x = dialog_client->x, req_y = dialog_client->y, 
-	    req_w = dialog_client->width, req_h = dialog_client->height;
-	  
-	  if (!dialog_constrain_geometry(dialog_client, 
-					 &req_x, &req_y, 
-					 &req_w, &req_h))
-	    {
-	      dialog_client->x = req_x; dialog_client->y = req_y; 
-	      dialog_client->width = req_w; dialog_client->height = req_h;
-	      dialog_client->move_resize(dialog_client);
-	      client_deliver_config(dialog_client);
-	    }
-	  
-	}
-    }
-
   c->y = w->dpy_height - wm_get_offsets_size(w, SOUTH, c, True) - c->height;
   c->x = wm_get_offsets_size(w, WEST,  NULL, False);
   c->width = w->dpy_width
     - wm_get_offsets_size(w, WEST,  NULL, False)
     - wm_get_offsets_size(w, EAST,  NULL, False);
+
+  /*
+   * Though not transient for app, there could be a fullscreened
+   * app win below us, in which case the input win needs to cover
+   * any vertical panels. 
+   */
+  if ((main_client = wm_get_visible_main_client(w)) != NULL)
+    {
+      if (main_client->flags & CLIENT_FULLSCREEN_FLAG)
+	{
+	  c->x      = 0;
+	  c->y      = w->dpy_height - c->height;
+	  c->width  = w->dpy_width;
+	}
+    }
+
+  if (c->flags & CLIENT_TB_ALT_TRANS_FOR_DIALOG)
+    {
+      Client *dialog_client = c->trans;
+      
+      dbg("%s() client trans for dialog\n", __func__);
+
+      if (dialog_client)
+	{
+	  /*
+	   *  Move transient dialog out of the way of toolbar.  
+	   */
+	  Bool tmp_mapped = c->mapped;
+
+	  int req_x = dialog_client->x, req_y = dialog_client->y, 
+	    req_w = dialog_client->width, req_h = dialog_client->height;
+	  
+	  c->mapped = True; 	/* Hack Hack */
+	  c->type = MBCLIENT_TYPE_TOOLBAR; 
+
+	  dbg("%s() checking for available geom\n", __func__);
+
+	  if (!dialog_constrain_geometry(dialog_client, 
+					 &req_x, &req_y, 
+					 &req_w, &req_h))
+	    {
+
+	      dbg("%s() constraining to %ix%i +%i+%i\n", 
+		  __func__,
+		  req_w, req_h, req_x, req_y);
+
+	      dialog_client->x = req_x; dialog_client->y = req_y; 
+	      dialog_client->width = req_w; dialog_client->height = req_h;
+	      dialog_client->move_resize(dialog_client);
+	      client_deliver_config(dialog_client);
+	    }
+
+	  c->type = MBCLIENT_TYPE_DIALOG; 
+	  c->mapped = tmp_mapped;
+	  
+	}
+    }
 }
 
 void
@@ -142,6 +188,9 @@ toolbar_client_move_resize(Client *c)
 
   base_client_move_resize(c);
 
+  dbg("%s() called setting size +%i+%i,%ix%i\n", 
+      __func__, c->x, c->y, c->width, c->height);
+
   XResizeWindow(w->dpy, c->window, c->width, c->height);
   XMoveResizeWindow(w->dpy, c->frame, c->x, c->y, c->width, c->height );
 }
@@ -192,6 +241,11 @@ toolbar_client_show(Client *c)
 	  /* resize the main client were transient for */
 	  
 	  Client *app_client = c->trans;
+
+	  /* Call this so, map of toolbar hopefully happens before
+           * resize preventing potential flash of desktop win. 
+	  */
+	  XSync(w->dpy, False);
 	  
 	  if (app_client)
 	    {
@@ -212,7 +266,6 @@ toolbar_client_hide(Client *c)
 {
   Wm *w = c->wm;
   
-  
   client_set_state(c, WithdrawnState);
   XReparentWindow(w->dpy, c->window, w->root, 0, 0); 
   XUnmapWindow(w->dpy, c->window);
@@ -235,12 +288,19 @@ toolbar_client_destroy(Client *c)
     {
       Client *app_client = c->trans;
 
-      if (app_client) /* possible crasher here, should check still exists */
+      if (app_client)
 	{
+	  /* app_client could have likely dissapeared with the toolbar
+	   * but we havn't been told yet - therefore trap. 
+           * XXX - there maybe a better way of handling this ?
+	  */
+	  misc_trap_xerrors(); 
+
 	  app_client->height += c->height;
 	  app_client->move_resize(app_client);
 	  app_client->redraw(app_client, False);
-	  /* XXX Send config ? */
+
+	  misc_untrap_xerrors();
 	}
     }
 
